"""
Copyright (c) 2004, CherryPy Team (team@cherrypy.org)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the CherryPy Team nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""


"""
A module containing a few utility classes/functions used by CherryPy
"""

import time, thread, cpg

def _cpLogMessage(msg, context = '', severity = 0):
    """ Default method for logging messages """

    nowTuple = time.localtime(time.time())
    nowStr = '%04d/%02d/%02d %02d:%02d:%02d' % (nowTuple[:6])
    if severity == 0: level = "INFO"
    elif severity == 1: level = "WARNING"
    elif severity == 2: level = "ERROR"
    try:
        logToScreen = int(cpg.parsedConfigFile.get('server', 'logToScreen'))
    except:
        logToScreen = 1
    if logToScreen:
        print nowStr, context, level, msg

def _cpOnError():
    """ Default _cpOnError method """

    import traceback, StringIO
    bodyFile = StringIO.StringIO()
    traceback.print_exc(file = bodyFile)
    cpg.response.body = bodyFile.getvalue()
    cpg.response.headerMap['Content-Type'] = 'text/plain'
    if cpg.request.isXmlRpc:
        # Special case for XML-RPC:
        cpg.response.body = xmlrpclib.dumps(xmlrpclib.Fault(1, response.body))
        cpg.response.headerMap['Content-Type'] = 'text/xml'

    cpg.response.headerMap["Content-Type"] = "text/plain"

def _cpInitThread(numThread): pass
def _cpInitRequest(): pass
def _cpInitResponse(): pass
def _cpInitRequestBeforeParse(): pass

def _cpSaveSessionData(sessionId, sessionData, expirationTime):
    """ Save session to file if needed """

    t = time.localtime(expirationTime)
    if cpg.configOption.sessionStorageType == 'file':
        fname=os.path.join(_sessionStorageFileDir,sessionId)
        if _threadPool>1 or _threading:
            cpg._sessionFileLock.acquire()
        f = open(fname,"wb")
        cPickle.dump((sessionData, expirationTime), f)
        f.close()
        if cpg.configOption._threadPool > 1 or cpg.configOption._threading:
            cpg._sessionFileLock.release()

    elif cpg.configOption.sessionStorageType=="ram":
        # Update expiration time
        cpg._sessionMap[sessionId] = (sessionData, expirationTime)

    elif cpg.configOption.sessionStorageType == "cookie":
        
        # TODO: cookie storage type is probably broken ...
        """ TODO: set siteKey in _cpConfig
            # Get site key from config file or compute it
            try: cpg._SITE_KEY_ = configFile.get('server','siteKey')
            except:
                _SITE_KEY_ = ''
                for i in range(30):
                    _SITE_KEY_ += whrandom.choice(string.letters)
        """

        # Update expiration time
        sessionData = (sessionData, expirationTime)
        dumpStr = cPickle.dumps(_sessionData)
        try: dumpStr = zlib.compress(dumpStr)
        except: pass # zlib is not available in all python distros
        dumpStr = binascii.hexlify(dumpStr) # Need to hexlify it because it will be stored in a cookie
        cpg.response.simpleCookie['CSession'] = dumpStr
        cpg.response.simpleCookie['CSession-sig'] = md5.md5(dumpStr + cpg.configOption.siteKey).hexdigest()
        cpg.response.simpleCookie['CSession']['path'] = '/'
        cpg.response.simpleCookie['CSession']['max-age'] = sessionTimeout * 60
        cpg.response.simpleCookie['CSession-sig']['path'] = '/'
        cpg.response.simpleCookie['CSession-sig']['max-age'] = sessionTimeout * 60

def _cpLoadSessionData(sessionId):
    """ Return the session data for a given sessionId.
        The _expirationTime will be checked by the caller of this function
    """
    if cpg.configOption.sessionStorageType == "ram":
        if cpg._sessionMap.has_key(sessionId):
            return cpg._sessionMap[sessionId]
        else: return None

    elif cpg.configOption.sessionStorageType == "file":
        fname = os.path.join(cpg.configOption.sessionStorageFileDir, sessionId)
        if os.path.exists(fname):
            if cpg.configOption.threadPool > 1 or cpg.configOption.threading:
                cpg._sessionFileLock.acquire()
            f = open(fname, "rb")
            sessionData = cPickle.load(f)
            f.close()
            if cpg.configOption.threadPool > 1 or cpg.configOption.threading:
                cpg._sessionFileLock.release()
            return sessionData
        else: return None

    elif _sessionStorageType == "cookie":
        if request.simpleCookie.has_key('CSession') and request.simpleCookie.has_key('CSession-sig'):
            data = request.simpleCookie['CSession'].value
            sig  = request.simpleCookie['CSession-sig'].value
            if md5.md5(data + cpg.configOption.siteKey).hexdigest() == sig:
                try:
                    dumpStr = binascii.unhexlify(data)
                    try: dumpStr = zlib.decompress(dumpStr)
                    except: pass # zlib is not available in all python distros
                    dumpStr = cPickle.loads(dumpStr)
                    return dumpStr
                except: pass
        return None

