"""
Copyright (c) 2004, CherryPy Team (team@cherrypy.org)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the CherryPy Team nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import cpg, urllib, sys, time, traceback, types, StringIO, cgi, os
import mimetypes, sha, whrandom, string, _cputil, cperror

"""
Common Service Code for CherryPy
"""

mimetypes.types_map['.dwg']='image/x-dwg'
mimetypes.types_map['.ico']='image/x-icon'

weekdayname = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
monthname = [None, 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']

def parseFirstLine(data):
    cpg.request.path = data.split()[1]
    if cpg.request.path and cpg.request.path[0] == '/':
        cpg.request.path = cpg.request.path[1:] # Remove starting '/' if any
    cpg.request.path = cpg.request.path.replace('&amp;', '&') # This case happens for some reason ...
    cpg.request.browserUrl = cpg.request.path
    cpg.request.paramMap = {}
    cpg.request.filenameMap = {}
    cpg.request.fileTypeMap = {}
    cpg.request.paramTuple = ()
    cpg.request.isXmlRpc = 0
    i = cpg.request.path.find('?')
    if i != -1:
        # Parse parameters from URL
        if cpg.request.path[i+1:]:
            k = cpg.request.path[i+1:].find('?')
            if k != -1:
                j = cpg.request.path[:k].rfind('=')
                if j != -1:
                    cpg.request.path = cpg.request.path[:j+1] + \
                        urllib.quote_plus(cpg.request.path[j+1:])
            for paramStr in cpg.request.path[i+1:].split('&'):
                sp = paramStr.split('=')
                if len(sp) > 2:
                    j = paramStr.find('=')
                    sp = (paramStr[:j], paramStr[j+1:])
                if len(sp) == 2:
                    key, value = sp
                    value = urllib.unquote_plus(value)
                    if cpg.request.paramMap.has_key(key):
                        # Already has a value: make a list out of it
                        if type(cpg.request.paramMap[key]) == type([]):
                            # Already is a list: append the new value to it
                            cpg.request.paramMap[key].append(value)
                        else:
                            # Only had one value so far: start a list
                            cpg.request.paramMap[key] = [cpg.request.paramMap[key], value]
                    else:
                        cpg.request.paramMap[key] = value
        cpg.request.path = cpg.request.path[:i]

    if cpg.request.path and cpg.request.path[-1] == '/':
        cpg.request.path = cpg.request.path[:-1] # Remove trailing '/' if any

def parsePostData(rfile):
    # Read request body and put it in data
    len = int(cpg.request.headerMap.get("Content-Length","0"))
    if len: data = rfile.read(len)
    else: data=""

    cpg.request.isXmlRpc=0
    # Try to parse request body as an XML-RPC call
    if 'xmlRpc' in cpg.configOption.typeOfRequests and data and cpg.request.headerMap.get("Content-Type","") == "text/xml":
        xmlRpcPathList = []
        if cpg.request.path == 'RPC2': pass
        elif cpg.request.path.find('/') > -1: xmlRpcPathList = cpg.request.path.split('/')
        elif not cpg.request.path: pass
        else: xmlRpcPathList = [request.path]
        try:
            try: cpg.request.paramTuple,thisXmlRpcMethod=xmlrpclib.loads(data)
            except: raise "XML-RPC ERROR"
            thisXmlRpcMethod = str(thisXmlRpcMethod) # Get rid of unicode
            cpg.request.isXmlRpc=1 # If parsing worked, it is an XML-RPC request
            xmlRpcPathList += thisXmlRpcMethod.split('.')
        except "XML-RPC ERROR":
            # error reading data; must not have been an xmlrpc file
            pass

    if cpg.request.isXmlRpc:
        cpg.request.path = '/'.join(xmlRpcPathList)
    else:
        # It's a normal browser call
        # Put data in a StringIO so FieldStorage can read it
        newRfile = StringIO.StringIO(data)
        forms = cgi.FieldStorage(fp = newRfile, headers = cpg.request.headerMap, environ = {'REQUEST_METHOD':'POST'}, keep_blank_values = 1)
        for key in forms.keys():
            # Check if it's a list or not
            valueList = forms[key]
            if type(valueList) == type([]):
                # It's a list of values
                cpg.request.paramMap[key] = []
                cpg.request.filenameMap[key] = []
                cpg.request.fileTypeMap[key] = []
                for item in valueList:
                    cpg.request.paramMap[key].append(item.value)
                    cpg.request.filenameMap[key].append(item.filename)
                    cpg.request.fileTypeMap[key].append(item.type)
            else:
                # It's a single value
                # In case it's a file being uploaded, we save the filename in a map (user might need it)
                cpg.request.paramMap[key] = valueList.value
                cpg.request.filenameMap[key] = valueList.filename
                cpg.request.fileTypeMap[key] = valueList.type

def _normalizeKey(key):
    """ Turn a HTTP header key into its "normalized value"
        For instance, "content-length" -> "Content-Length"
    """
    newKey = ''
    for i in range(len(key)):
        if i == 0 or key[i-1] == '-':
            newKey += key[i].upper()
        else:
            newKey += key[i]
    return newKey

def insertIntoHeaderMap(key,value):
    cpg.request.headerMap[_normalizeKey(key)] = value

def doRequest(wfile):
    try:
        handleRequest(wfile)
    except:
        err = ""
        exc_info_1 = sys.exc_info()[1]
        if hasattr(exc_info_1, 'args') and len(exc_info_1.args) >= 1:
            err = exc_info_1.args[0]

        try:
            _cputil.getSpecialFunction('_cpOnError')()
            wfile.write('%s %s\r\n' % (cpg.response.headerMap['protocolVersion'], cpg.response.headerMap['Status']))
            if cpg.response.headerMap.has_key('Content-Length') and cpg.response.headerMap['Content-Length'] == 0:
                cpg.response.headerMap['Content-Length'] = len(cpg.response.body)
            for key, valueList in cpg.response.headerMap.items():
                if key not in ('Status', 'protocolVersion'):
                    if type(valueList) != type([]): valueList = [valueList]
                    for value in valueList:
                        wfile.write('%s: %s\r\n'%(key, value))
            wfile.write('\r\n')
            wfile.write(cpg.response.body)
        except:
            bodyFile = StringIO.StringIO()
            traceback.print_exc(file = bodyFile)
            body = bodyFile.getvalue()
            wfile.write('%s 200 OK\r\n' % cpg.configOption.protocolVersion)
            wfile.write('Content-Type: text/plain\r\n')
            wfile.write('Content-Length: %s\r\n' % len(body))
            wfile.write('\r\n')
            wfile.write(body)

def sendResponse(wfile):
    # Save page in the cache if needed
    cacheKey = ""
    try: cacheKey = cpg.request.cacheKey # Cannot use "hasattr", because it fails under jython when threading is turned on
    except: pass
    if cacheKey:
        cacheMap[cpg.request.cacheKey] = (cpg.request.cacheExpire, cpg.response.headerMap, cpg.response.body)

    # Save session data
    if cpg.configOption.sessionStorageType and not cpg.request.isStatic:
        sessionId = cpg.response.simpleCookie[cpg.configOption.sessionCookieName].value
        expirationTime = time.time() + cpg.configOption.sessionTimeout * 60
        _cputil.getSpecialFunction('_cpSaveSessionData')(sessionId, cpg.request.sessionMap, expirationTime)

    wfile.write('%s %s\r\n' % (cpg.response.headerMap['protocolVersion'], cpg.response.headerMap['Status']))
    for key, valueList in cpg.response.headerMap.items():
        if key not in ('Status', 'protocolVersion'):
            if type(valueList) != type([]): valueList = [valueList]
            for value in valueList:
                wfile.write('%s: %s\r\n'%(key, value))
    # Send response cookies
    cookie = cpg.response.simpleCookie.output()
    # print "Sending back cookie:", cookie
    if cookie:
        wfile.write(cookie+'\r\n')
    wfile.write('\r\n')
    wfile.write(cpg.response.body)

def sendCachedPageIfPossible(now, wfile):
    cacheKey = ""
    try: cacheKey = cpg.request.cacheKey # Cannot use "hasattr", because it fails under jython when threading is turned on
    except: pass
    if cacheKey:
        # Use caching for this page
        if cacheMap.has_key(cpg.request.cacheKey) and cacheMap[cpg.request.cacheKey][0] >= now:
            # This page is already in the cache and it hasn't expired yet: use the cached version
            dummy, cpg.response.headerMap, cpg.response.body = cacheMap[cpg.request.cacheKey]
            _cputil.getSpecialFunction('_cpInitResponse')()
            cpg.request.cacheKey='' # No need to save the page in the cache again
            sendResponse(wfile)
            return 1
        # else:
        #     Either the page has never been cached, or the cached version expired
        #     In the case, we build the page normally and it will be saved in the cache
        #    in the sendResponse function
    return 0

def handleRequest(wfile):
    now = time.time()
    year, month, day, hh, mm, ss, wd, y, z = time.gmtime(now)
    date = "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (weekdayname[wd], day, monthname[month], year, hh, mm, ss)
    cpg.response.headerMap={"Status": "200 OK", "protocolVersion": cpg.configOption.protocolVersion, "Content-Type": "text/html", "Server": "CherryPy/" + cpg.__version__, "Date": date, "Set-Cookie": [], "Content-Length": 0}

    # Two variables used for streaming
    cpg.response.wfile = wfile
    cpg.response.sendResponse = 1

    if cpg.configOption.sslKeyFile:
        cpg.request.base = "https://" + cpg.request.headerMap['Host']
    else:
        cpg.request.base = "http://" + cpg.request.headerMap['Host']
    cpg.request.browserUrl = cpg.request.base + '/' + cpg.request.browserUrl
    cpg.request.isStatic = 0

    # Flush the cache if needed:
    if cpg.configOption.flushCacheDelay and cpg.cacheMap and cpg.lastCacheFlushTime + cpg.configOption.flushCacheDelay * 60 <= now:
        lastCacheFlushTime = now
        for key, (expire, dummy, dummy) in cacheMap.items():
            if expire <= now:
                del cacheMap[key]

    # Clean up expired sessions if needed:
    if cpg.configOption.sessionStorageType and cpg.configOption.sessionCleanUpDelay and cpg._lastSessionCleanUpTime + cpg.configOption.sessionCleanUpDelay * 60 <= now:
        cpg._lastSessionCleanUpTime = now
        ramOrFileOrCookieOrCustomCleanUpOldSessions()

    # Perform some initial operations (such as rewriting url ...)
    cpg.request.originalPath = cpg.request.path
    cpg.request.originalParamMap = cpg.request.paramMap
    cpg.request.originalParamTuple = cpg.request.paramTuple

    _cputil.getSpecialFunction('_cpInitRequest')()

    path = cpg.request.path

    # Handle static directories
    for urlDir, fsDir in cpg.configOption.staticContentList:
        if path == urlDir or path[:len(urlDir)+1]==urlDir+'/':

            cpg.request.isStatic = 1

            if sendCachedPageIfPossible(now, wfile): return

            fname = fsDir + path[len(urlDir):]
            stat = os.stat(fname)
            if type(stat) == type(()): # Python2.1
                modifTime = stat[9]
            else:
                modifTime = stat.st_mtime

            strModifTime = time.strftime("%a, %d %b %Y %H:%M:%S GMT", time.gmtime(modifTime))

            # Check if browser sent "if-modified-since" in request header
            if cpg.request.headerMap.has_key('If-Modified-Since'):
                # Check if if-modified-since date is the same as strModifTime
                if cpg.request.headerMap['If-Modified-Since'] == strModifTime:
                    cpg.response.headerMap = {'Status': 304, 'protocolVersion': cpg.configOption.protocolVersion, 'Date': date}
                    cpg.response.body = ''
                    _cputil.getSpecialFunction('_cpInitResponse')()
                    sendResponse(wfile)
                    return

            cpg.response.headerMap['Last-Modified'] = strModifTime
            f=open(fname, 'rb')
            cpg.response.body = f.read()
            cpg.response.headerMap['Content-Length'] = len(cpg.response.body)
            f.close()
            # Set content-type based on filename extension
            i = path.rfind('.')
            if i != -1: ext = path[i:]
            else: ext = ""
            contentType = mimetypes.types_map.get(ext, "text/plain")
            cpg.response.headerMap['Content-Type'] = contentType
            _cputil.getSpecialFunction('_cpInitResponse')()
            sendResponse(wfile)
            return

    # Get session data
    if cpg.configOption.sessionStorageType and not cpg.request.isStatic:
        now = time.time()
        # First, get sessionId from cookie
        try: sessionId = cpg.request.simpleCookie[cpg.configOption.sessionCookieName].value
        except: sessionId=None
        if sessionId:
            # Load session data from wherever it was stored
            sessionData = _cputil.getSpecialFunction('_cpLoadSessionData')(sessionId)
            if sessionData == None:
                sessionId = None
            else:
                cpg.request.sessionMap, expirationTime = sessionData
                # Check that is hasn't expired
                if now > expirationTime:
                    # Session expired
                    sessionId = None

        # Create a new sessionId if needed
        if not sessionId:
            cpg.request.sessionMap = {}
            sessionId = generateSessionId()
            cpg.request.sessionMap['_sessionId'] = sessionId

        cpg.response.simpleCookie[cpg.configOption.sessionCookieName] = sessionId
        cpg.response.simpleCookie[cpg.configOption.sessionCookieName]['path'] = '/'
        cpg.response.simpleCookie[cpg.configOption.sessionCookieName]['version'] = 1

    # if sendCachedPageIfPossible(now, wfile): return # TODO: re-implement caching

    path = cpg.request.path

    # Traverse path:
    # for /a/b, we'll try:
    #   root.a.b.index()
    #   root.a.b.default()

    #   root.a.b()
    #   root.a.default()

    #   root.default()

    # Also, a method has to have ".exposed = True" in order to be exposed

    if not path:
        pathList = []
    elif cpg.request.isXmlRpc:
        pathList = path.split('.')
    else:
        pathList = path.split('/')

    pathList = ['root'] + pathList

    # try root.a.b, then root.a, then root
    func = None

    obj = cpg
    previousObj = None
    objList = []
    searchedPathList = []
    myPath = ''
    bestKnownDefaultMethod = None
    # Successively get objects from the path: 'root', then 'a' then 'b'
    for pathItem in pathList:
        previousObj = obj
        try:
            # find contained object
            obj = getattr(obj, pathItem)

            # add object
            objList.append(obj)
            searchedPathList.append(pathItem)

            # if found object has a default method, remember it for later
            default = getattr(obj, 'default', None)
            if default:
                # TODO: check if default is callable?
                bestKnownDefaultMethod = default
                bestKnownDefaultMethodMyPath = '/'.join(searchedPathList[1:])

        except AttributeError:
            break

    # TODO: the following code could probably be KISSed somewhat.

    if len(objList) == len(pathList):
        # root_a_b exists
        root_a_b = objList[-1]

        # Try root.a.b.index()
        root_a_b_dot_index = getattr(root_a_b, 'index', None)
        if root_a_b_dot_index and getattr(root_a_b_dot_index, 'exposed', None):
            myPath = '/'.join(pathList[1:])
            func = root_a_b_dot_index
        else:
            # Try root.a.b.default()
            root_a_b_dot_default = getattr(root_a_b, 'default', None)
            if root_a_b_dot_default and getattr(root_a_b_dot_default, 'exposed', None):
                # XXX: I don't think this ever gets called?
                myPath = 'root_a_b_dot_default'
                func = root_a_b_dot_default
            elif callable(root_a_b) and getattr(root_a_b, 'exposed', None):
                # We use root.a.b()
                myPath = '/'.join(pathList[1:-1])
                func = root_a_b

    if func == None:
        # None of these exist: use the default method we found earlier
        if bestKnownDefaultMethod:
            func = bestKnownDefaultMethod
            myPath = bestKnownDefaultMethodMyPath

    if func == None:
        raise cperror.NotFound

    if cpg.request.isXmlRpc:
        cpg.response.body = func(*(cpg.request.paramTuple))
    else:
        # XXX
        myPath += '/'
        if len(myPath) > 1:
            myPath = '/' + myPath

        cpg.request.objectPath = myPath
        cpg.request.virtualPath = cpg.request.path[len(myPath)-1:]
        cpg.response.body = func(**(cpg.request.paramMap))

    _cputil.getSpecialFunction('_cpInitResponse')()

    if cpg.request.isXmlRpc:
        # Marshall the result if it's an XML-RPC call
        # Wrap the response into a singleton tuple
        cpg.response.body = (response.body,)
        cpg.response.body = xmlrpclib.dumps(response.body, methodresponse=1)
        # Response type is text/xml for an XML-RPC call
        cpg.response.headerMap["Content-Type"]="text/xml"

    # check if results are generators
    # don't forget that python2.1 doesn't have generators
    if hasattr(types, 'GeneratorType') and  type(cpg.response.body) == types.GeneratorType:
        # makes a list of the generator. it's easier but less efficient than to
        # loop over the iterator while writing to the wfile. This code does not
        # check if all lines are strings.
        cpg.response.body = "".join(list(cpg.response.body))

    # we don't support Unicode strings
    elif type(cpg.response.body) == types.UnicodeType: # Potential gotcha: on jython, type("") == type(u"") !!
        raise "CherryError: The mask or view returned a unicode string instead of a regular string!"

    # if it's something different than a string, bail out
    elif type(cpg.response.body) != types.StringType:
        raise "CherryError: The method didn't return a string!"
    
    if cpg.response.headerMap.has_key('Content-Length') and cpg.response.headerMap['Content-Length']==0:
        cpg.response.headerMap['Content-Length']=len(cpg.response.body)

    if cpg.response.sendResponse: sendResponse(wfile)


def generateSessionId():
    s = ''
    for i in range(50):
        s += whrandom.choice(string.letters+string.digits)
    s += '%s'%time.time()
    return sha.sha(s).hexdigest()

