"""
Copyright (c) 2004, CherryPy Team (team@cherrypy.org)
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
    * Neither the name of the CherryPy Team nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import os,urllib,time,sys,signal,socket,httplib

def startServer(infoMap):
    # Start the server in another thread
    if not hasattr(os, "fork"): # win32 mostly
        pid=os.spawnl(os.P_NOWAIT, infoMap['path'], infoMap['path'], 'testsite.py')
    else:
        pid=os.fork()
        if not pid:
            os.execlp(infoMap['path'],infoMap['path'],'testsite.py')
    return pid

def getPage(url, cookies, isSSL=0, extraRequestHeader = []):
    data=""
    i=0
    response = None
    while i<10:
        try:
            if isSSL:
                conn=httplib.HTTPSConnection('127.0.0.1:8000')
            else:
                conn=httplib.HTTPConnection('127.0.0.1:8000')

            conn.putrequest("GET", url)

            conn.putheader("Host", "127.0.0.1")
            if cookies:
                cookieList = []
                for cookie in cookies:
                    i = cookie.find(' ')
                    j = cookie.find(';')
                    cookieList.append(cookie[i+1:j])
                cookieStr = '; '.join(cookieList)
                conn.putheader("Cookie", cookies[:j])

            for key, value in extraRequestHeader:
                conn.putheader(key, value)

            conn.endheaders()

            response=conn.getresponse()

            cookies=response.msg.getallmatchingheaders("Set-Cookie")

            data=response.read()

            conn.close()
            break
        except socket.error:
            time.sleep(0.5)
        i+=1
    return data, cookies, response

def getXmlrpc(url, func, isSSL=0):
    import xmlrpclib
    http="http"
    if isSSL: http+="s"
    if url: url='/'+url
    data=""
    i=0
    try:
        while i<10:
            try:
                testsvr=xmlrpclib.Server(http+"://127.0.0.1:8000"+url)
                data=eval("testsvr.%s"%func)
                break
            except socket.error:
                time.sleep(0.5)
            i+=1
    except xmlrpclib.Fault, msg:
        return msg
    return data


def shutdownServer(pid, mode, isSSL=0):
    if isSSL: h="https"
    else: h="http"
    if mode=='t':
        u=urllib.urlopen(h+"://127.0.0.1:8000/shutdown/thread")
        if hasattr(socket, 'sslerror'): sslError = socket.sslerror
        else: sslError = 'dummy'
        try: u=urllib.urlopen(h+"://127.0.0.1:8000/shutdown/dummy")
        except IOError: pass
        except sslError: pass
        except AttributeError: pass # Happens on Mac OS X when run with Python-2.3
    elif mode=='tp':
        try: sslError = socket.sslerror
        except: sslError = 'dummy'
        u=urllib.urlopen(h+"://127.0.0.1:8000/shutdown/thread")
        try: u=urllib.urlopen(h+"://127.0.0.1:8000/shutdown/dummy")
        except IOError: pass # Happens on Windows
        except sslError: pass # Happens on Windows for https requests
    else:
        try:
            u=urllib.urlopen(h+"://127.0.0.1:8000/shutdown/regular")
        except IOError: pass
        except AttributeError: pass # For Python2.3

def checkResult(testName, infoMap, serverMode, result, expectedResult, failedList, exactResult):
    if result==expectedResult or (exactResult==0 and result.find(expectedResult)!=-1): return 1
    else:
        failedList.append(testName+" for python%s"%infoMap['exactVersionShort']+" in "+serverMode+" mode failed: expected result was %s, result was %s"%(`expectedResult`, `result`))
        f=open('result', 'w')
        f.write("Real result:"+ repr(result))
        f.close()
        return 0

def prepareCode(code):
    f = open('testsite.py', 'w')
    beforeStart = '''
class Shutdown:
    def dummy(self):
        return "OK"
    dummy.exposed = 1
    def regular(self):
        import os
        os._exit(0)
    regular.exposed = 1
    def thread(self):
        import threading
        for t in threading.enumerate(): t.setName("NOT RUNNING")
        return "OK"
    thread.exposed = 1
cpg.root.shutdown = Shutdown()
def f(*a, **kw): return ""
cpg.root._cpLogMessage = f
'''
    f.write(code.replace('cpg.server.start', beforeStart + 'cpg.server.start'))
    f.close()

def checkPageResult(testName, infoMap, code, config, urlList, expectedResultList, failedList, exactResult=1, isSSL=0, extraRequestHeader=[], expectedHeaderList=[]):
    response = None
    prepareCode(code)
    # Try it in all 3 modes (regular, threading, threadPooling) (we're missing forking and process pooling)
    modeList=[('r',''), ('tp', 'threadPool=3')]
    # modeList=[('r','')] # TODO
    for mode,modeConfig in modeList:
        f=open("testsite.cfg", "w")
        f.write(config)
        f.write('''
[session]
storageType=ram
[server]
socketPort = 8000
''')
        f.write(config+"\n"+modeConfig)
        f.close()

        pid = startServer(infoMap)
        passed=1
        cookies=None
        for i in range(len(urlList)):
            url=urlList[i]
            expectedResult=expectedResultList[i]
            result, cookies, response=getPage(url, cookies, isSSL, extraRequestHeader)
            if expectedHeaderList:
                if response.status != expectedHeaderList[0]:
                    failedList.append(testName+" for python%s"%infoMap['exactVersionShort']+" in "+mode+" mode failed: expected result status was %s, result status was %s"%(expectedHeaderList[0], response.status))
                    passed=0
                    print "*** FAILED ***"
                    break
            if not checkResult(testName, infoMap, mode, result, expectedResult, failedList, exactResult):
                passed=0
                print "*** FAILED ***"
                break
        shutdownServer(pid, mode, isSSL)
        if passed:
            print mode+"...",
            sys.stdout.flush()
        else: break
    if passed: print "passed"
    sys.stdout.flush()
    return response

def checkCompilerNoError(testName, infoMap, code, extraCompileOption, failedList):
    res=compileCode(infoMap, code, extraCompileOption)
    if res.lower().find('error')!=-1:
        failedList.append(testName+" for python%s"%infoMap['exactVersionShort']+' failed: compilation failed (%s)'%res)
        print "*** FAILED ***"
    else: print "passed"    

def checkInCompilerError(testName, infoMap, code, extraCompileOption, compilerErrorList, failedList):
    res=compileCode(infoMap, code, extraCompileOption)
    for compilerError in compilerErrorList:
        if res.find(compilerError)==-1:
            failedList.append(testName+" for python%s"%infoMap['exactVersionShort']+' failed: expected compilation error "%s", but compilation returned "%s"'%(compilerError, res))
            print "*** FAILED ***"
            return
    print "passed"

def checkNotInSourceFile(testName, infoMap, code, extraCompileOption, errorCodeList, failedList):
    compiledCode=getCompiledCode(testName, infoMap, code, extraCompileOption, failedList)
    if not compiledCode: return
    for errorCode in errorCodeList:
        if compiledCode.find(errorCode)!=-1:
            failedList.append(testName+" for python%s"%infoMap['exactVersionShort']+' failed: found "%s" in compiled code'%errorCode)
            print "*** FAILED ***"
            return
    print "passed"

def checkInSourceFile(testName, infoMap, code, extraCompileOption, codeList, failedList):
    compiledCode=getCompiledCode(testName, infoMap, code, extraCompileOption, failedList)
    if not compiledCode: return
    for code in codeList:
        if compiledCode.find(code)==-1:
            failedList.append(testName+" for python%s"%infoMap['exactVersionShort']+' failed: "%s" not found in compiled code'%code)
            print "*** FAILED ***"
            return
    print "passed"

def checkXmlrpcResult(testName, infoMap, code, config, extraCompileOption, urlList, funcList, expectedResultList, failedList, exactResult=1, isSSL=0):
    res=compileCode(infoMap, code, extraCompileOption)
    if res.lower().find('error')!=-1:
        failedList.append(testName+" for python%s"%infoMap['exactVersionShort']+' failed: compilation failed (%s)'%res)
        print "*** FAILED ***"
    else:
        # Try it in all 3 modes (regular, threading, threadPooling) (we're missing forking and process pooling)
        modeList=[('r',''), ('t', 'threading=1'), ('tp', 'threadPool=3')]
        for mode,modeConfig in modeList:
            f=open("testsite.cfg", "w")
            f.write(config+"\n"+modeConfig)
            f.close()

            pid=startServer(infoMap)
            passed=1
            cookies=None
            for i in range(len(urlList)):
                url=urlList[i]
                func=funcList[i]
                expectedResult=expectedResultList[i]
                result=getXmlrpc(url, func, isSSL)
                if not checkResult(testName, infoMap, mode, result, expectedResult, failedList, exactResult):
                    passed=0
                    print "*** FAILED ***"
                    break
            shutdownServer(pid, mode, isSSL)
            if passed:
                print mode+"...",
                sys.stdout.flush()
            else: break
        if passed: print "passed"
        sys.stdout.flush()


